#' Plot time profile clusters of dynamic analysis result.
#' 
#' @param fuzzed_matsplines result of dynamic analysis: inferred net generated 
#' by dynamicConsensusNet function.
#' @param ... further plotting/legend parameters.
#' @return pdf file in current working directory.
#' 
#' @keywords manip
#' @export
#' @examples
#' #please run with whole database files (prepared according to vignette)
#' data(OmicsExampleData)
#' data_omics = readOmics(tp_prots = c(0.25, 1, 4, 8, 13, 18, 24), 
#' tp_genes = c(1, 4, 8, 13, 18, 24), OmicsExampleData,
#' PWdatabase = c("biocarta", "kegg", "nci", "reactome"), 
#' TFtargetdatabase = c("chea", "pazar"))
#' \dontrun{
#' data_omics = readTFdata(data_omics)
#' data_omics_plus = readPWdata(data_omics, 
#' loadgenelists = "Genelists")
#' data_omics = identifyPWs(data_omics_plus)
#' data_omics = identifyTFs(data_omics)
#' data_omics = enrichPWs(data_omics)
#' data_omics = identifyRsofTFs(data_omics, only_enriched = FALSE, 
#' noTFs_inPW = 1, order_neighbors = 10)
#' data_omics = identifyPWTFTGs(data_omics, only_enriched = FALSE)
#' statConsNet = staticConsensusNet(data_omics)
#' dynConsNet = dynamicConsensusNet(data_omics, statConsNet)
#' timeprof = clusterTimeProfiles(dynConsNet)
#' plotTimeProfileClusters(timeprof)
#' }
plotTimeProfileClusters <- function(fuzzed_matsplines) {
    
    requireNamespace("Mfuzz", quietly = TRUE)
    pdf(paste(getwd(), "/", "TimeProfiles.pdf", sep = ""))
    par(mar=c(5.1, 4.1, 4.1, 8.1), xpd = TRUE )
    plot(x = NULL, y = NULL, bty = 'L', 
         xlim = c(min(as.numeric(colnames(fuzzed_matsplines$centers))), 
                  max(as.numeric(colnames(fuzzed_matsplines$centers)))+1),
         ylim = c(-2,2), xlab = "time", ylab = "normalized expression", 
         main = paste("Fuzzy c-means clustering\n with ", 
                      max(fuzzed_matsplines$cluster), " centers", sep = ""))
    colors = rainbow(max(fuzzed_matsplines$cluster))
    ind_sort = order(fuzzed_matsplines$cluster)
    
    for(s in 1: max(fuzzed_matsplines$cluster))
    {lines(as.numeric(colnames(fuzzed_matsplines$centers)), 
           fuzzed_matsplines$centers[s,], col = colors[s])}
    if(length(names(fuzzed_matsplines$cluster)[ind_sort]) < 20)
    {legend("topright", inset = c(-0.3,0), 
            legend = names(fuzzed_matsplines$cluster)[ind_sort], 
            fill = colors[fuzzed_matsplines$cluster][ind_sort], cex = 0.7)
    }else{legend("topright", inset = c(-0.3,0), 
                 legend = paste("cluster ", 1:dim(fuzzed_matsplines$centers)[1],
                                sep = ""), 
                 fill = colors[1:dim(fuzzed_matsplines$centers)[1]], cex = 0.7)        
    }
    dev.off()
}

#' Plot inferred net based on analysis analysis.
#' 
#' Dynamic analysis result is plotted to pdf file stored in current working 
#' directory.
#'  
#' @param dynConsensusNet result of dynamic analysis: inferred net generated by 
#' dynamicConsensusNet function.
#' @param sig.level significance level for plotting edges in network 
#' (between 0 and 1).
#' @param clarify indicating if unconnected nodes should be removed; 
#' default = "TRUE".
#' @param layout igraph layout parameter; default is 
#' layout.fruchterman.reingold.
#' @param ... further plotting/legend parameters.
#' @return pdf file in current working directory.
#' 
#' @keywords manip
#' @export
#' @examples
#' #please run with whole database files (prepared according to vignette)
#' data(OmicsExampleData)
#' data_omics = readOmics(tp_prots = c(0.25, 1, 4, 8, 13, 18, 24), 
#' tp_genes = c(1, 4, 8, 13, 18, 24), OmicsExampleData,
#' PWdatabase = c("biocarta", "kegg", "nci", "reactome"), 
#' TFtargetdatabase = c("chea", "pazar"))
#' \dontrun{
#' data_omics = readTFdata(data_omics)
#' data_omics_plus = readPWdata(data_omics,  
#' loadgenelists = FALSE)
#' data_omics = identifyPWs(data_omics_plus)
#' data_omics = identifyTFs(data_omics)
#' data_omics = enrichPWs(data_omics)
#' data_omics = identifyRsofTFs(data_omics, only_enriched = FALSE, 
#' noTFs_inPW = 1, order_neighbors = 10)
#' data_omics = identifyPWTFTGs(data_omics, only_enriched = FALSE)
#' statConsNet = staticConsensusNet(data_omics)
#' dynConsNet = dynamicConsensusNet(data_omics, statConsNet)
#' plotdynConsensusNet(dynConsNet, sig.level = 0.8)
#' }
plotdynConsensusNet <- function(dynConsensusNet, sig.level, clarify = "TRUE",
                                layout = layout.fruchterman.reingold, ...) {
    
    dynConsNet = dynConsensusNet[[1]]
    pdf(paste(getwd(), "/", "dynConsensusNet.pdf", sep = ""))
    dynnet_z = t(dynConsNet$z)
    dynnet <- matrix(0, nrow = nrow(dynnet_z), ncol = ncol(dynnet_z))
    colnames(dynnet) <- colnames(dynnet_z)
    rownames(dynnet) <- rownames(dynnet_z)
    cutoff <- qnorm((1 + sig.level)/2)
    if(dim(which(abs(dynnet_z) > cutoff, arr.ind = TRUE))[1] == 0)
    { stop("No nodes in the network have a significance higher than the
            selected value. Please choose a less strict significance level.\n")}
    dynnet[which(abs(dynnet_z) > cutoff, arr.ind = TRUE)] <- 1
    remove_ind <- which(colSums(dynnet) == 0 & rowSums(dynnet) == 0)
    if(length(remove_ind) >= dim(dynnet)[1]-1)
    { stop("Please select clarify = 'FALSE' to have network contents visualized
           and/or select a less strict significance level.\n")}
    if (clarify == "TRUE" & length(remove_ind) > 0) 
    {dynnet <- dynnet[-remove_ind, -remove_ind]}
    dyngraph <- graph.adjacency(dynnet, mode = c("directed"), 
                                add.rownames = TRUE)
    ident = unlist(strsplit(as.character(V(dyngraph)$name),"_"))[seq(2, length(V(dyngraph)$name)*2, by = 2)]
    V(dyngraph)$color[which(ident == "p")] = "red"
    V(dyngraph)$color[which(ident == "g")] = "green"
    V(dyngraph)$name = unlist(strsplit(V(dyngraph)$name, "_"))[seq(1, length(ident)*2, by = 2)]
    V(dyngraph)$label.cex = 0.6
    plot(dyngraph, main = "Dynamic consensus net", layout = layout,...)
    legend("topleft" , legend = c("consensus proteins", "consensus genes"), 
           fill = c("red", "green"), cex = 0.7, ...)
    dev.off()
}

#' Plot consensus graph(s) from static analysis.
#' 
#' Consensus graph(s) plotted to pdf file stored in current working directory.
#'  
#' @param consensusGraphs result from static analysis: consensus graph generated 
#' by staticConsensusNet function.
#' @param data_omics OmicsData object.
#' @param ... further plotting/legend parameters.
#' @return pdf file in current working directory.
#' 
#' @keywords manip
#' @export
#' @examples
#' #please run with whole database files (prepared according to vignette)
#' data(OmicsExampleData)
#' data_omics = readOmics(tp_prots = c(0.25, 1, 4, 8, 13, 18, 24), 
#' tp_genes = c(1, 4, 8, 13, 18, 24), OmicsExampleData,
#' PWdatabase = c("biocarta", "kegg", "nci", "reactome"), 
#' TFtargetdatabase = c("chea", "pazar"))
#' \dontrun{
#' data_omics = readTFdata(data_omics)
#' data_omics_plus = readPWdata(data_omics,  
#' loadgenelists = FALSE)
#' data_omics = identifyPWs(data_omics_plus)
#' data_omics = identifyTFs(data_omics)
#' data_omics = enrichPWs(data_omics)
#' data_omics = identifyRsofTFs(data_omics, only_enriched = FALSE, 
#' noTFs_inPW = 1, order_neighbors = 10)
#' data_omics = identifyPWTFTGs(data_omics, only_enriched = FALSE)
#' statConsNet = staticConsensusNet(data_omics)
#' plot(statConsNet)
#' }
plotConsensusGraph <- function(consensusGraphs, data_omics, ...) {
    
    if(class(data_omics) != "OmicsData")
    {stop("Parameter 'data_omics' is not an OmicsData object.")}
    
    if(TRUE %in% !(names(consensusGraphs) %in% 
                       as.character(data_omics[[1]][[1]][[1]][[1]]) &
                       names(consensusGraphs) %in% 
                       as.character(data_omics[[1]][[1]][[1]][[2]])) ) 
    {warning("Warning: ConsensusGraphs do not match OmicsData object.")}
    
    pdf(paste(getwd(), "/", "ConsensusGraphs.pdf", sep = ""))
    same_tps = data_omics[[1]][[1]][[1]][[1]][which(data_omics[[1]][[1]][[1]][[1]] 
                                                    %in% data_omics[[1]][[1]][[1]][[2]])]
    for(k in 1: length(consensusGraphs))
    {
        V(consensusGraphs[[k]])$label.cex = 0.6
        plot.igraph(consensusGraphs[[k]], 
                    main = paste("Consensus graph\n time ", 
                                 same_tps[k], sep = ""), vertex.size = 18, ...)
        legend("topleft" , legend = c("consensus proteins", 
                                      "steiner node proteins", "consensus TFs", "consensus target genes"), 
               fill = c("red", "yellow", "lightblue", "green"), cex = 0.7, ...)
    }
    dev.off()
}

